@@ -1,3 +1,9 @@
+/*
+ * Avoid conflicts with existing md5 code by building names in a unique
+ * fashion.
+ */
+#define MD5NS(x) webbasemd5_##x
+
 /* Define if you have the z library (-lz).  */
 #undef HAVE_LIBZ
 
@@ -31,6 +37,18 @@
 /* Define if tables must be split using MERGE to unite them.  */
 #undef TABLE_SPLIT
 
+/* BSD style signals ? */
+#undef HAVE_BSD_SIGNALS
+
+/* Posix style signals ? */
+#undef HAVE_POSIX_SIGNALS
+
+/* USG style signals ? */
+#undef HAVE_USG_SIGHOLD
+
+/* Defined if signal handler returns void */
+#undef VOID_SIGHANDLER
+
 @TOP@
 
 /* Type of second argument of getsockname */
