@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 1997, 1998, 1999, 2000
+ *   Copyright (C) 1997, 1998, 1999, 2000, 2001
  *   	Free Software Foundation, Inc.
  *
  *   This program is free software; you can redistribute it and/or modify it
@@ -24,6 +24,7 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
+#include <signal.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
@@ -36,6 +37,7 @@
 
 #include <getopttools.h>
 #include <crawl.h>
+#include <crawlsig.h>
 #include <webbase.h>
 #include <webbase_create.h>
 #include <sqlutil.h>
@@ -75,11 +77,14 @@ static int verbose = 0;
 static void init(int argc, char** argv);
 static void finish();
 static void crawlers(int argc, char** argv);
+static void signal_setup();
 
 int main(int argc, char** argv)
 {
   crawl_params_t* crawl;
 
+  signal_setup();
+
   init(argc, argv);
 
   if(params.schema) {
@@ -88,6 +93,8 @@ int main(int argc, char** argv)
   }
 
   crawl = params.crawl;
+  if(!params.create)
+    server_load_re(crawl->servers);
 
   if(params.home_pages) {
     hp_load_1(crawl, params.where_start);
@@ -344,7 +351,7 @@ static void crawlers(int argc, char** argv)
     int pids_list[2];
     int pid_count = 0;
     if(verbose) fprintf(stderr, "exploring %d entries from table start\n", num_rows);
-    while((row = mysql_fetch_row(res))) {
+    while((row = mysql_fetch_row(res)) && !interrupted) {
       processed_rows++; 
       if(!row[0]) {
 	fprintf(stderr, "crawlers: unexpected null row[0]\n");
@@ -435,6 +442,23 @@ static void crawlers(int argc, char** argv)
     }
   }
 
+  /*
+   * Forward signal to children
+   */
+  if(interrupted) {
+    int i;
+    for(i = 0; i < params.crawlers_nproc; i++) {
+      if(child_pids[i] > 0) {
+	if(kill(child_pids[i], SIGINT) < 0) {
+	  if(errno != ESRCH) {
+	    fprintf(stderr, "unable to kill child with pid = %d ", child_pids[i]);
+	    perror("");
+	  }
+	}
+      }
+    }
+  }
+  
   /*
    * Wait for all children to die.
    */
@@ -458,3 +482,50 @@ static void crawlers(int argc, char** argv)
   free(argv_child);
 }
 
+#if !defined (RETSIGTYPE)
+#  if defined (VOID_SIGHANDLER)
+#    define RETSIGTYPE void
+#  else
+#    define RETSIGTYPE int
+#  endif /* !VOID_SIGHANDLER */
+#endif /* !RETSIGTYPE */
+
+#if defined (VOID_SIGHANDLER)
+#  define SIGHANDLER_RETURN return
+#else
+#  define SIGHANDLER_RETURN return (0)
+#endif
+
+static RETSIGTYPE signal_interrupt(int sig)
+{
+  struct sigaction action_interrupt;
+  memset((char*)&action_interrupt, '\0', sizeof(struct sigaction));
+
+  if(verbose) printf("caught signal %d\n", sig);
+  /* 
+   * Globaly inform the program that a signal occured and it must
+   * die gracefully.
+   */
+  interrupted = 1;
+
+  /*
+   * Ignore all further signals, let the program die in peace.
+   */
+  action_interrupt.sa_handler = SIG_IGN;
+  sigaction(SIGINT, &action_interrupt, NULL);
+  sigaction(SIGHUP, &action_interrupt, NULL);
+  sigaction(SIGTERM, &action_interrupt, NULL);
+
+  SIGHANDLER_RETURN;
+}
+
+static void signal_setup()
+{
+  struct sigaction action_interrupt;
+  memset((char*)&action_interrupt, '\0', sizeof(struct sigaction));
+
+  action_interrupt.sa_handler = signal_interrupt;
+  sigaction(SIGINT, &action_interrupt, NULL);
+  sigaction(SIGHUP, &action_interrupt, NULL);
+  sigaction(SIGTERM, &action_interrupt, NULL);
+}
