@@ -1,19 +1,19 @@
 /*
- *   Copyright (C) 1997, 1998, 1999 Loic Dachary
+ * Copyright (C) 1997 - 2001 Loic Dachary
  *
- *   This program is free software; you can redistribute it and/or modify it
- *   under the terms of the GNU General Public License as published by the
- *   Free Software Foundation; either version 2, or (at your option) any
- *   later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 /*
@@ -36,7 +36,7 @@
 #include <dmalloc.h>
 #endif /* HAVE_DMALLOC_H */
 
-#include <hash.h>
+#include <khash.h>
 #include <getopttools.h>
 #include <cookies.h>
 #include <mysql.h>
@@ -439,7 +439,7 @@ static void cookies_decode(cookies_t* cookies, MYSQL_RES *, MYSQL_ROW row)
 void cookies_free(cookies_t* params)
 {
   cookies_save_entry(params);
-  hash_free(params->options);
+  _K(hash_free)(params->options);
   free(params);
 }
 
