@@ -1,19 +1,19 @@
 /*
- *   Copyright (C) 1997, 1998, 1999, 2000 Loic Dachary
+ * Copyright (C) 1997 - 2001 Loic Dachary
  *
- *   This program is free software; you can redistribute it and/or modify it
- *   under the terms of the GNU General Public License as published by the
- *   Free Software Foundation; either version 2, or (at your option) any
- *   later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 /*
@@ -69,6 +69,7 @@
 #endif /* LANGREC */
 #include <webbasedl.h>
 #include <webbase_hook.h>
+#include <crawlsig.h>
 
 static int verbose = 1;
 
@@ -215,6 +216,7 @@ struct option* crawl_options(struct option options[])
   getopt_merge(options, webtools_options(options));
   getopt_merge(options, robots_options(options));
   getopt_merge(options, cookies_options(options));
+  getopt_merge(options, server_options(options));
 
   return long_options;
 }
@@ -225,6 +227,7 @@ struct option_help* crawl_help_options(struct option_help options_help[])
   getopt_help_merge(options_help, webtools_help_options(options_help));
   getopt_help_merge(options_help, robots_help_options(options_help));
   getopt_help_merge(options_help, cookies_help_options(options_help));
+  getopt_help_merge(options_help, server_help_options(options_help));
 
   return long_options_help;
 }
@@ -397,6 +400,12 @@ crawl_params_t* crawl_alloc(int argc, char** argv, struct option options[])
    */
   params->robots = robots_alloc(argc, argv, options);
   params->robots->base = params->base;
+  /*
+   * Create server name handling module
+   */
+  params->servers = server_alloc(argc, argv, options);
+  params->servers->base = params->base;
+  params->base->servers = params->servers;
   /*
    * Create the inverted index module
    */
@@ -404,7 +413,7 @@ crawl_params_t* crawl_alloc(int argc, char** argv, struct option options[])
     params->hook = (webbase_hook_params_t*)(*hook_init)();
     params->hook->getopt(params->hook, argc, argv);
     params->hook->init(params->hook, params->base);
-    server_hook(params->hook);
+    server_hook(params->servers, params->hook);
     if(verbose) fprintf(stderr, "loaded hook library %s\n", params->hook->name ? params->hook->name : "(unknown)");
   }
   if(params->candidate_limit <= 0)
@@ -454,7 +463,7 @@ static void crawl_rebuild_1(char* paramsp, webbase_url_t* webbase_url)
     if(file_exists(path))
 #endif /* WEBBASE_CONTENT_BASE */
       {
-	params->hook->server(params->hook, server_id(params->base, webbase_url->w_url));
+	params->hook->server(params->hook, server_id(params->servers, webbase_url->w_url));
 	params->hook->insert(params->hook, webbase_url);
 	if(verbose > 1) fprintf(stderr, "\trebuild %s (%d)\n", webbase_url->w_url, webbase_url->w_hookid);
 	sprintf(query, "update url set hookid = %d where rowid = %d\n", webbase_url->w_hookid, webbase_url->w_rowid);
@@ -535,7 +544,7 @@ static void crawl_urls_1(char* arg, MYSQL_RES* , MYSQL_ROW row)
 }
 
 /*
- * Reload <urls> even if not needed according to heuristics.
+ * Reload <urls>.
  */
 void crawl_urls(crawl_params_t* params, char* urls)
 {
@@ -642,7 +651,8 @@ void hp_remove_unloaded(crawl_params_t* params, char* url)
 }
 
 /*
- * Crawl url alone ignoring heuristics.
+ * Crawl url alone, reloading only if content of document is not 
+ * there.
  */
 webbase_url_t* crawl_touch(crawl_params_t* params, char* url)
 {
@@ -703,11 +713,11 @@ void hp_load_in_core(crawl_params_t* params, char* url)
   do {
     params->robot_delayed = 0;
     hp_load_mirror((char*)params, &start);
-  } while(params->robot_delayed > 0);
+  } while(params->robot_delayed > 0 && !interrupted);
 #ifdef WEBBASE_LOCK
   webbase_unlock(base, start.rowid, "start");
 #endif /* WEBBASE_LOCK */
-  if(verbose) fprintf(stderr, "exploration done\n");
+  if(verbose) fprintf(stderr, "exploration %s\n", (interrupted ? "interrupted" : "done"));
 }
 
 /*
@@ -845,7 +855,7 @@ void hp_unload(crawl_params_t* params, char* url, int keep_start)
       sprintf(query, "update start set info = 0, count = 0 where rowid = %d", start.rowid);
     } else {
       sprintf(query, "delete from start where rowid = %d", start.rowid);
-      server_remove(params->base, url);
+      server_remove(params->servers, url);
     }
     smysql_query(&base->mysql, query);
   }
@@ -1054,7 +1064,7 @@ void hp_load_1(crawl_params_t* params, char* where)
     params->robot_delayed = 0;
     if(verbose > 1) fprintf(stderr, "\thp_load_1: %s\n", robots_delay_where);
     webbase_walk_start(params->base, robots_delay_where, hp_load_mirror, (char*)params);
-    if(params->robot_delayed) {
+    if(params->robot_delayed && !interrupted) {
       if(verbose) fprintf(stderr, "still %d home pages delayed\n", params->robot_delayed);
       if(first) {
 	if(where && where[0] != '\0') {
@@ -1065,9 +1075,9 @@ void hp_load_1(crawl_params_t* params, char* where)
 	first = 0;
       }
     }
-  } while(params->robot_delayed);
+  } while(params->robot_delayed && !interrupted);
   free(robots_delay_where);
-  if(verbose) fprintf(stderr, "exploration done\n");
+  if(verbose) fprintf(stderr, "exploration %s\n", (interrupted ? "interrupted" : "done"));
 }
 
 static void noheuristics_1(char* contextp, MYSQL_RES* , MYSQL_ROW row)
@@ -1537,7 +1547,7 @@ static void mirror_explore_new(crawl_context_t* context)
 
   do {
     mirror_query(context, query);
-  } while(!context->empty);
+  } while(!context->empty && !interrupted);
 
   {
     int state = context->start->delay ? WEBBASE_URL_START_EXPLORING : WEBBASE_URL_START_EXPLORED;
@@ -1562,11 +1572,11 @@ static int mirror_query(crawl_context_t* context, char* query)
   num_rows = mysql_num_rows(res);
   if(num_rows > 0) {
     if(verbose > 1) fprintf(stderr, "\t%d candidate urls\n", num_rows);
-    while((row = mysql_fetch_row(res)) && !context->empty) {
+    while((row = mysql_fetch_row(res)) && !context->empty && !interrupted) {
       char* url = row[0];
 
       robots_load(context->params->robots, context, url);
-      if(context->params->hook) context->params->hook->server(context->params->hook, server_id(context->params->base, url));
+      if(context->params->hook) context->params->hook->server(context->params->hook, server_id(context->params->servers, url));
       if((webbase_url = mirror(context, url))) {
 	/*
 	 * walk_href returns the status of the last 'mirror_crawl_continue'
@@ -1579,7 +1589,7 @@ static int mirror_query(crawl_context_t* context, char* query)
 	}
       }
     }
-    if(verbose > 1) fprintf(stderr, "\texploration of %s %s\n", context->start->url, (context->empty ? "interrupted" : "done"));
+    if(verbose > 1) fprintf(stderr, "\texploration of %s %s\n", context->start->url, ((context->empty || interrupted) ? "interrupted" : "done"));
   } else {
     if(verbose > 1) fprintf(stderr, "\tquery returns empty set\n");
     context->empty = 1;
@@ -1838,14 +1848,14 @@ static webbase_url_t* mirror_2(crawl_context_t* context, char* url)
 #else /* TABLE_SPLIT */
     sprintf(query, "select rowid from url_content where rowid = %d", webbase_url->w_rowid);
 #endif /* TABLE_SPLIT */
-    if(!sql_select_exists(&context->base->mysql, query)) {
+    if(context->params->noheuristics == 1 || !sql_select_exists(&context->base->mysql, query)) {
       if(verbose > 1) fprintf(stderr, "\tmirror: touching\n");
       webbase_url->w_crawl = time(0) - WEBBASE_TIME_DAY;
     }
 #else /* WEBBASE_CONTENT_BASE */
     char* path = webbase_url_file(webbase_url->w_rowid);
     if(path) {
-      if(!file_exists(path)) {
+      if(context->params->noheuristics == 1 || !file_exists(path)) {
 	if(verbose > 1) fprintf(stderr, "\tmirror: touching\n");
 	webbase_url->w_crawl = time(0) - WEBBASE_TIME_DAY;
       }
@@ -2465,7 +2475,7 @@ static char* mirror_request_http(crawl_context_t* context, webbase_url_t* webbas
   /*
    * Fill If-Modified-Since field if appropriate.
    */
-  if(!context->touch && (webbase_url->w_mtime > 0)) {
+  if(context->params->noheuristics == 0 && (webbase_url->w_mtime > 0)) {
     sprintf(modified_since, "If-Modified-Since: %s\r\n", time2str(webbase_url->w_mtime));
     if(verbose > 1) fprintf(stderr, "\tmirror_request_http: %.*s\n", (int)(strlen(modified_since) - 2), modified_since);
   }
@@ -2553,6 +2563,7 @@ void crawl_free(crawl_params_t* params)
   dirsel_end();
   robots_free(params->robots);
   cookies_free(params->cookies);
+  server_free(params->servers);
   _K(hash_free)(params->options);
   webbase_free(params->base);
   webtools_free(params->http);
