@@ -27,6 +27,7 @@
 #include <http.h>
 #include <robots.h>
 #include <cookies.h>
+#include <server.h>
 #include <webbase_hook.h>
 
 #define CRAWL_PARAMS_CANDIDATE_LIMIT	200
@@ -38,6 +39,7 @@ typedef struct crawl_params {
   webbase_url_start_t start_in_core;
   robots_t* robots;
   cookies_t* cookies;
+  server_t* servers;
   int robot_delayed;
   int noheuristics;
   int update;
