@@ -25,11 +25,7 @@
 #include "ftp.h"
 #endif
 
-#if defined(_WIN32)
-#define SETSOCKOPT_OPTVAL_TYPE (const char *)
-#else
-#define SETSOCKOPT_OPTVAL_TYPE (void *)
-#endif
+#define SETSOCKOPT_OPTVAL_TYPE (SETSOCKOPT_ARG3)
 
 #define FTP_BUFSIZ 8192
 #define ACCEPT_TIMEOUT 30
@@ -42,7 +38,7 @@ typedef struct {
 } netbuf;
 
 #if 0
-static char *rcs_id = "$Id: ftp.cc,v 1.4 2000/01/26 11:26:38 loic Exp $";
+static char *rcs_id = "$Id: ftp.cc,v 1.5 2000/06/15 17:28:13 loic Exp $";
 #endif
 static int sDatal = -1;
 static int sControl;
