@@ -21,16 +21,17 @@
 
 #include <webbase.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
+/*
+ * Map extension to mime type
+ */
 char* mime_ext2mime(webbase_t* base, char* path, char* default_type);
+/*
+ * Return true if extension is accepted, false otherwise
+ */
 int mime_accept_ext(webbase_t* base, char* spec, char* path);
+/*
+ * Return true if mime type is accepted, false otherwise
+ */
 int mime_accept_type(webbase_t* base, char* spec, char* content_type);
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif _mime_h
