@@ -29,7 +29,7 @@
 #include <dmalloc.h>
 #endif /* HAVE_DMALLOC_H */
 
-#include <hash.h>
+#include <khash.h>
 #include <getopttools.h>
 #include <uri.h>
 #include <salloc.h>
@@ -257,9 +257,9 @@ static int robots_load_1(robots_t* robots, crawl_context_t* context, uri_t* url_
     }
 
     /*
-       * Query the base to get the information just inserted. This
-       * is not optimal but prevent duplicating code from robots_decode.
-       */
+     * Query the base to get the information just inserted. This
+     * is not optimal but prevent duplicating code from robots_decode.
+     */
     sprintf(query, query_format, netloc);
     smysql_query(&base->mysql, query);
     res = smysql_store_result(&base->mysql);
@@ -349,7 +349,7 @@ void robots_free(robots_t* params)
   robots_save_entry(params);
   if(params->current.allow) free(params->current.allow);
   if(params->current.disallow) free(params->current.disallow);
-  hash_free(params->options);
+  _K(hash_free)(params->options);
   free(params);
 }
 
