@@ -211,12 +211,12 @@ webbase_t* webbase_alloc(int argc, char** argv, struct option options[])
 
   base->pid = getpid();
   base->lock_wait = 60;
-  if(getenv("ECILA_LOCK_WAIT")) {
-    base->lock_wait = atoi(getenv("ECILA_LOCK_WAIT"));
+  if(getenv("WEBBASE_LOCK_WAIT")) {
+    base->lock_wait = atoi(getenv("WEBBASE_LOCK_WAIT"));
   }
   base->lock_max_loop = 200;
-  if(getenv("ECILA_LOCK_MAX_LOOP")) {
-    base->lock_max_loop = atoi(getenv("ECILA_LOCK_MAX_LOOP"));
+  if(getenv("WEBBASE_LOCK_MAX_LOOP")) {
+    base->lock_max_loop = atoi(getenv("WEBBASE_LOCK_MAX_LOOP"));
   }
 
   if(verbose > 1) fprintf(stderr, "webbase_alloc: finished\n");
@@ -229,6 +229,7 @@ webbase_t* webbase_alloc(int argc, char** argv, struct option options[])
 
 void webbase_free(webbase_t* base)
 {
+  base->servers = 0; /* We do not created this object, we do not delete it. */
   _K(hash_free)(base->options);
   mysql_close(&base->mysql);
   if(base->mysql_unix_port) free(base->mysql_unix_port);
@@ -241,10 +242,10 @@ void webbase_lock_ignore(webbase_t*, int varint, char* varchar)
   static char name[512];
 
   if(varchar) {
-    sprintf(name, "ECILA_IGNORE=%d%s", varint, varchar);
+    sprintf(name, "WEBBASE_IGNORE=%d%s", varint, varchar);
     if(verbose > 1) fprintf(stderr, "\tset lock_ignored \n");
   } else {
-    strcpy(name, "ECILA_IGNORE=");
+    strcpy(name, "WEBBASE_IGNORE=");
     if(verbose > 1) fprintf(stderr, "\tunset lock_ignored \n");
   }
 
@@ -275,7 +276,7 @@ int webbase_lock(webbase_t* base, int varint, char* varchar)
   name_length = strlen(name);
   
   {
-    char* ignore = getenv("ECILA_IGNORE");
+    char* ignore = getenv("WEBBASE_IGNORE");
     if(ignore && !strcmp(name, ignore)) {
       if(verbose) fprintf(stderr, "\tlock ignored %s\n", name);
       return 1;
@@ -350,7 +351,7 @@ void webbase_unlock(webbase_t* base, int varint, char* varchar)
   name_length = strlen(name);
 
   {
-    char* ignore = getenv("ECILA_IGNORE");
+    char* ignore = getenv("WEBBASE_IGNORE");
     if(ignore && !strcmp(name, ignore)) {
       if(verbose) fprintf(stderr, "\tlock ignored %s\n", name);
       return ;
@@ -640,7 +641,7 @@ void webbase_update_start(webbase_t* base, webbase_url_start_t* start)
     sprintf(query + strlen(query), " where rowid = %d", start->rowid);
     if(verbose > 5) fprintf(stderr, "update start: %s\n", query);
   } else {
-    server_add(base, start->url);
+    server_add(base->servers, start->url);
     sprintf(query, "replace into start ("WEBBASE_START_FIELDS_INSERT") values (%d, '%s', '%s', %d, ",
 	    start->rowid,
 	    sql_quote_char(&url, &url_size, start->url, strlen(start->url)),
