@@ -1,19 +1,19 @@
 /*
- *   Copyright (C) 1997, 1998, 1999 Loic Dachary
+ * Copyright (C) 1997 - 2001 Loic Dachary
  *
- *   This program is free software; you can redistribute it and/or modify it
- *   under the terms of the GNU General Public License as published by the
- *   Free Software Foundation; either version 2, or (at your option) any
- *   later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 #ifndef _webbase_h
@@ -33,6 +33,8 @@
 
 #define WEBBASE_OPTIONS		0x0004000
 
+typedef struct server server_t;
+
 typedef struct webbase {
   /* Public */
   hash_t* options;
@@ -54,6 +56,8 @@ typedef struct webbase {
   int lock_wait;
   /* Number of retries on a locked resource */
   int lock_max_loop;
+  /* Server map */
+  server_t* servers;
 
   hash_t* query;
 
