@@ -1,30 +1,29 @@
 /*
- *   Copyright (C) 1997, 1998, 1999 Loic Dachary
+ * Copyright (C) 1997 - 2001 Loic Dachary
  *
- *   This program is free software; you can redistribute it and/or modify it
- *   under the terms of the GNU General Public License as published by the
- *   Free Software Foundation; either version 2, or (at your option) any
- *   later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <stdlib.h>
 #ifdef HAVE_DMALLOC_H
 #include <dmalloc.h>
 #endif /* HAVE_DMALLOC_H */
-#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
@@ -542,6 +541,7 @@ create table locks ( \n\
 "insert into mime2ext values ('application/x-pkcs7-mime', 'p7m,p7c,') ",
 "insert into mime2ext values ('application/x-pkcs7-signature', 'p7s,') ",
 "insert into mime2ext values ('application/x-rar-compressed', 'rar,r00,r01,r02,r03,r04,r05,r06,r07,r08,r09,r10,r11,r12,r13,r14,r15,r16,r17,r18,r19,r20,r21,r22,r23,r24,r25,r26,r27,r28,r29,r30,r31,r32,r33,r34,r35,r36,r37,r38,r39,r40,r41,r42,r43,r44,r45,r46,r47,r48,r49,r50,r51,r52,r53,r54,r55,r56,r57,r58,r59,r60,r61,r62,r63,r64,r65,r66,r67,r68,r69,r70,r71,r72,r73,r74,r75,r76,r77,r78,r79,r80,r81,r82,r83,r84,r85,r86,r87,r88,r89,r90,r91,r92,r93,r94,r95,r96,r97,r98,r99,') ",
+"insert into mime2ext values ('application/x-rpm', 'rpm,') ",
 "insert into mime2ext values ('application/x-shar', 'shar,') ",
 "insert into mime2ext values ('application/x-sh', 'sh,') ",
 "insert into mime2ext values ('application/x-stuffit', 'sit,') ",
