@@ -1,19 +1,19 @@
 /*
- *   Copyright (C) 1997, 1998, 1999, 2000 Loic Dachary
+ * Copyright (C) 1997 - 2001 Loic Dachary
  *
- *   This program is free software; you can redistribute it and/or modify it
- *   under the terms of the GNU General Public License as published by the
- *   Free Software Foundation; either version 2, or (at your option) any
- *   later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -34,7 +34,7 @@
 #include <dmalloc.h>
 #endif /* HAVE_DMALLOC_H */
 
-#include <hash.h>
+#include <khash.h>
 #include <split.h>
 #include <uri.h>
 #include <salloc.h>
@@ -361,6 +361,17 @@ void webbase_url_mtime_set(webbase_url_t* webbase_url, char* mtime)
   webbase_url->w_mtime = string2time(mtime);
 }
 
+/*
+ * The date is used for last modified time, only if a last modified time
+ * is not already available.
+ */
+void webbase_url_date_set(webbase_url_t* webbase_url, char* mtime)
+{
+  if(webbase_url->w_mtime == 0) {
+    webbase_url->w_mtime = string2time(mtime);
+  }
+}
+
 void webbase_url_location_set(webbase_url_t* webbase_url, char* location)
 {
   static uri_t* location_object = 0;
