@@ -1,19 +1,19 @@
 /*
- *   Copyright (C) 1997, 1998, 1999 Loic Dachary
+ * Copyright (C) 1997 - 2001 Loic Dachary
  *
- *   This program is free software; you can redistribute it and/or modify it
- *   under the terms of the GNU General Public License as published by the
- *   Free Software Foundation; either version 2, or (at your option) any
- *   later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -59,6 +59,7 @@
 #include <strshift.h>
 
 #include <webtools.h>
+#include <crawlsig.h>
 
 #define file_state_save \
   file->buffer = buffer; \
@@ -244,6 +245,7 @@ int webtools_reader(webtools_params_t* params, int sd)
 void webtools_close(webtools_params_t* params, int sd)
 {
   struct webtools_file* f = file(params, sd);
+  if(verbose) fprintf(stderr, "webtools_close: now freeing 0x%x for fd = %d\n", (int)f, sd);
   if(f) {
     if(f->buffer) free(f->buffer);
     f->buffer_length = 0;
@@ -286,6 +288,7 @@ static int webtools_reader_http_header(webtools_params_t* params, int sd, struct
   char* p;
   int code = WEBTOOLS_READER_OK;
   int done = 0;
+  int saw_interruption = 0;
 
   params->callback_http_header(params->callback_arg, sd, buffer, 0, WEBTOOLS_READER_START);
 
@@ -296,12 +299,22 @@ static int webtools_reader_http_header(webtools_params_t* params, int sd, struct
     if((bytes_read = read_timeout(sd, p, BUFFER_SIZE, params->timeout)) <= 0) {
       if(bytes_read == 0) {
 	code = WEBTOOLS_READER_HTTP_HEADER_SHORT;
+	done = 1;
       } else {
-	fprintf(stderr, "reading socket (header) : %d\n", errno);
-	code = -1;
+	/*
+	 * Interrupted by user signal, resume.
+	 */
+	if(errno == EINTR && !saw_interruption && interrupted) {
+	  saw_interruption = 1;
+	} else {
+	  fprintf(stderr, "reading socket (header) : %d\n", errno);
+	  code = -1;
+	  done = 1;
+	}
       }
-      done = 1;
-    } else {
+    }
+
+    if(!done && bytes_read > 0) {
       int i;
       if(verbose) fprintf(stderr, "webtools_reader_http_header: got %.*s\n", bytes_read, p);
       /*
@@ -358,6 +371,7 @@ static int webtools_reader_http_body(webtools_params_t* params, int sd, struct w
   int code = WEBTOOLS_READER_OK;
   int total_read = buffer_length;
   int done = 0;
+  int saw_interruption = 0;
 
   if(verbose) fprintf(stderr, "buffer_length = %d\n", buffer_length);
   static_alloc(&buffer, &buffer_size, buffer_length + BUFFER_SIZE + 1);
@@ -368,13 +382,24 @@ static int webtools_reader_http_body(webtools_params_t* params, int sd, struct w
   while(!done) {
     int bytes_read;
     if((bytes_read = read_timeout(sd, p, BUFFER_SIZE, params->timeout)) <= 0) {
-      params->callback_http_body(params->callback_arg, sd, buffer, buffer_length, WEBTOOLS_READER_END);
-      if(bytes_read < 0) {
-	fprintf(stderr, "reading socket (body) : %d\n", errno);
-	code = -1;
+      if(bytes_read == 0) {
+	params->callback_http_body(params->callback_arg, sd, buffer, buffer_length, WEBTOOLS_READER_END);
+	done = 1;
+      } else {
+	/*
+	 * Interrupted by user signal, resume.
+	 */
+	if(errno == EINTR && !saw_interruption && interrupted) {
+	  saw_interruption = 1;
+	} else {
+	  fprintf(stderr, "reading socket (body) : %d\n", errno);
+	  code = -1;
+	  done = 1;
+	}
       }
-      done = 1;
-    } else {
+    }
+
+    if(!done && bytes_read > 0) {
       buffer_length += bytes_read;
       total_read += bytes_read;
       if(verbose) fprintf(stderr, "buffer_length = %d\n", buffer_length);
@@ -459,4 +484,3 @@ static struct webtools_file* file(webtools_params_t* params, int sd)
     return 0;
   }
 }
-
