@@ -1,19 +1,19 @@
 /*
- *   Copyright (C) 2000 Loic Dachary
+ * Copyright (C) 2000 - 2001 Loic Dachary
  *
- *   This program is free software; you can redistribute it and/or modify it
- *   under the terms of the GNU General Public License as published by the
- *   Free Software Foundation; either version 2, or (at your option) any
- *   later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 
@@ -276,9 +276,13 @@ static int hook_insert_watch(webbase_hook_params_t* params)
 
 static int hook_update(webbase_hook_params_t* params, webbase_url_t* webbase_url)
 {
-  if(params->verbose) fprintf(stderr, "reindex %s\n", webbase_url->w_url);
-  hook_delete_id(params, webbase_url->w_hookid, webbase_url->w_url, strlen(webbase_url->w_url));
-  hook_insert(params, webbase_url);
+  if(webbase_url->w_code == WEBBASE_URL_CODE_NOT_MODIFIED) {
+    if(params->verbose) fprintf(stderr, "reindex %s (nothing changed)\n", webbase_url->w_url);
+  } else {
+    if(params->verbose) fprintf(stderr, "reindex %s\n", webbase_url->w_url);
+    hook_delete_id(params, webbase_url->w_hookid, webbase_url->w_url, strlen(webbase_url->w_url));
+    hook_insert(params, webbase_url);
+  }
   return 1;
 }
 
