@@ -19,9 +19,17 @@
 #ifndef _isomap_h
 #define _isomap_h
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 extern unsigned char iso2letter_unaccent[256];
 extern unsigned char iso2letter_transparent[256];
 
 unsigned char* unaccent(unsigned char* string, int string_length);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* isomap_h */
