@@ -23,6 +23,9 @@
 #ifdef HAVE_MALLOC_H
 #include <malloc.h>
 #endif /* HAVE_MALLOC_H */
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif /* HAVE_STDLIB_H */
 #ifdef HAVE_DMALLOC_H
 #include <dmalloc.h>
 #endif /* HAVE_DMALLOC_H */
@@ -54,11 +57,11 @@ char* srealloc(char* pointer, int size)
 void static_alloc(char** tmp, int* tmp_size, int new_size)
 {
   if(*tmp_size == 0) {
-    *tmp_size = 16;
+    *tmp_size = new_size + 16;
     *tmp = (char*)smalloc(*tmp_size);
   }
   if(*tmp_size < new_size) {
-    *tmp_size = new_size + 1;
+    *tmp_size = new_size + (new_size >= 2 ? new_size / 2 : 1);
     *tmp = (char*)srealloc(*tmp, *tmp_size);
   }
 }
