@@ -19,6 +19,14 @@
 #ifndef _strlower_h
 #define _strlower_h
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 void strlower(char* string, int string_length);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* strlower_h */
