@@ -19,6 +19,14 @@
 #ifndef _strncasecmp_h
 #define _strncasecmp_h
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 int strncasecmp (const char *a, const char *b, int n);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* strncasecmp_h */
