@@ -35,9 +35,10 @@ void time_register(time_t* timevar)
   }
 }
 
-void time_show(char* message, time_t timevar)
+time_t time_show(char* message, time_t timevar)
 {
   if(timevar == 0)
     timevar = time(0) - time_base;
   fprintf(stderr, "%s (%ldh:%ldm:%lds) \n", message, (timevar / 3600), ((timevar % 3600) / 60), (timevar % 60));
+  return timevar;
 }
