@@ -23,7 +23,7 @@
 extern "C" {
 #endif
 
-char* urldirname(char* filename);
+const char* urldirname(const char* filename);
 
 #ifdef __cplusplus
 }
